"""Weight calculation functions for moving averages."""

from .ema import ema_weights as ema_weights
from .geometric import geometric_weights as geometric_weights
from .logarithmic import logarithmic_weights as logarithmic_weights
