from mm_toolbox.candles.base cimport BaseCandles

cdef class PriceCandles(BaseCandles):
    cdef:
        double price_bucket
        double upper_price_bound
        double lower_price_bound
    # def __init__(
    #     self,
    #     double price_bucket,
    #     int num_candles=1000,
    #     bint store_trades=True
    # )

    cpdef void process_trade(self, object trade)
