"""Moving average calculation algorithms."""

from .ema import ExponentialMovingAverage as ExponentialMovingAverage
from .sma import SimpleMovingAverage as SimpleMovingAverage
from .tema import TimeExponentialMovingAverage as TimeExponentialMovingAverage
from .wma import WeightedMovingAverage as WeightedMovingAverage
