"""Candle aggregation and processing functionality."""

from .base import Trade as Trade, Candle as Candle
from .multi import MultiCandles as MultiCandles
from .price import PriceCandles as PriceCandles
from .tick import TickCandles as TickCandles
from .time import TimeCandles as TimeCandles
from .volume import VolumeCandles as VolumeCandles
